public Number3D sub2(Number3D v, Number3D w) {
  return new Number3D(  (v.x - w.x), (v.y - w.y), (v.z - w.z));
}

public Number3D add2(Number3D v, Number3D w) {
  return new Number3D(  (v.x + w.x), (v.y + w.y), (v.z + w.z));
}

public Number3D invert2(Number3D v) {
  return new Number3D( -v.x, -v.y, -v.z);
}


public Number3D copyFrom2(Number3D v) {
  return new Number3D( v.x, v.y, v.z);
}

public Number3D normalize2(Number3D v) {
  float m  = sqrt((v.x * v.x) + (v.y * v.y) + (v.z * v.z));
  return new Number3D( v.x/m, v.y/m, v.z/m );
}

/**
 * @param offsetParam Translation offset of axis
 * @param controlParam Regulates size and scale of radius
 * @param axisParam Regulates orientation rotation
 * @param angle Amount rotated
 **/
public Number3D rotateAroundArbitraryAxis(Number3D offsetParam,Number3D controlParam,Number3D axisParam,float angle) {
  Number3D control = copyFrom2(controlParam);
  Number3D axis = copyFrom2(axisParam);
  Number3D offset = copyFrom2(offsetParam);

  control.sub(offset);
  axis.sub(offset);
  axis.normalize(axis);

  float rads = PI/180 * angle;                       
  float sinTheta = sin(rads*.5);
  float cosTheta = cos(rads*.5);

  Quaternion rotQuat   = new Quaternion(  cosTheta,  
  sinTheta * axis.x,  
  sinTheta * axis.y,  
  sinTheta * axis.z );
  Quaternion pointQuat = new Quaternion(  0,
  control.x,
  control.y,
  control.z );
  Quaternion conjQuat  = new Quaternion(  rotQuat.a,
  -rotQuat.b,
  -rotQuat.c,
  -rotQuat.d );

  Quaternion temp = quat_mult(rotQuat,pointQuat);
  Number3D result = quat_pointmult(temp,conjQuat);
  result.add(offset);
  return result;
}

public Number3D rotateAroundZAxis(Number3D offset, Number3D controlParam, float angle) {

  Number3D control = copyFrom2(controlParam);
  control.sub(offset);

  float rads = PI/180 * angle;
  float cosTheta = cos(rads);
  float sinTheta = sin(rads);

  float x = cosTheta*control.x + sinTheta*control.y;
  float y = -sinTheta*control.x + cosTheta*control.y;
  float z = control.z;

  Number3D result = new Number3D(x,y,z);
  result.add(offset);

  return result;
}

public Number3D rotateAroundYAxis(Number3D offset, Number3D controlParam, float angle) {
  Number3D control = copyFrom2(controlParam);
  control.sub(offset);

  float rads = PI/180 * angle;
  float cosTheta = cos(rads);
  float sinTheta = sin(rads);

  float x = cosTheta*control.x - sinTheta*control.z;
  float y = control.y;
  float z = sinTheta*control.x + cosTheta*control.z;

  Number3D result = new Number3D(x,y,z);

  result.add(offset);

  return result;
}

public Number3D rotateAroundXAxis(Number3D offset, Number3D controlParam, float angle) {
  Number3D control = copyFrom2(controlParam);
  control.sub(offset);

  float rads = PI/180 * angle;
  float cosTheta = cos(rads);
  float sinTheta = sin(rads);

  float x = control.x;
  float y = cosTheta*control.y + sinTheta*control.z;
  float z = -sinTheta*control.y + cosTheta*control.z;

  Number3D result = new Number3D(x,y,z);

  result.add(offset);

  return result;
}

class Number3D {
  public float x=0;
  public float y=0;
  public float z=0;
  public Number3D next;

  public Number3D(float a,float b,float c) {
    this.x = a;
    this.y = b;
    this.z = c;
  }

  public void copyFrom(Number3D toCopy) {
    this.x = toCopy.x;
    this.y = toCopy.y;
    this.z = toCopy.z;
  }

  public void invert() {
    this.x = -this.x;
    this.y = -this.y;
    this.z = -this.z;
  }                      

  public void reset() {
    this.x = 0;
    this.y = 0;
    this.z = 0;
  }

  public void sub(Number3D v) {
    this.x -= v.x;
    this.y -= v.y;
    this.z -= v.z;
  }

  public void add(Number3D v) {
    this.x += v.x;
    this.y += v.y;
    this.z += v.z;
  }

  public void normalize(Number3D v) {
    float m  = sqrt((v.x * v.x) + (v.y * v.y) + (v.z * v.z));

    this.x = v.x/m;
    this.y = v.y/m;
    this.z = v.z/m;
  }
}


